@@ -22,7 +22,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
-#include <unistd.h> 
+#include <unistd.h>
 #include <time.h>
 #include <assert.h>
 #ifdef HAVE_LOCALE_H
@@ -147,19 +147,6 @@ gpgsm_print_time (estream_t fp, ksba_isotime_t t)
 }
 
 
-void
-gpgsm_dump_time (ksba_isotime_t t)
-{
-  if (!t || !*t)
-    log_printf (_("[none]"));
-  else
-    log_printf ("%.4s-%.2s-%.2s %.2s:%.2s:%s",
-                t, t+4, t+6, t+9, t+11, t+13);
-}
-
-
-
-
 void
 gpgsm_dump_string (const char *string)
 {
@@ -188,7 +175,7 @@ gpgsm_dump_string (const char *string)
 
 
 /* This simple dump function is mainly used for debugging purposes. */
-void 
+void
 gpgsm_dump_cert (const char *text, ksba_cert_t cert)
 {
   ksba_sexp_t sexp;
@@ -196,7 +183,7 @@ gpgsm_dump_cert (const char *text, ksba_cert_t cert)
   char *dn;
   ksba_isotime_t t;
 
-  log_debug ("BEGIN Certificate `%s':\n", text? text:"");
+  log_debug ("BEGIN Certificate '%s':\n", text? text:"");
   if (cert)
     {
       sexp = ksba_cert_get_serial (cert);
@@ -207,11 +194,11 @@ gpgsm_dump_cert (const char *text, ksba_cert_t cert)
 
       ksba_cert_get_validity (cert, 0, t);
       log_debug ("  notBefore: ");
-      gpgsm_dump_time (t);
+      dump_isotime (t);
       log_printf ("\n");
       ksba_cert_get_validity (cert, 1, t);
       log_debug ("   notAfter: ");
-      gpgsm_dump_time (t);
+      dump_isotime (t);
       log_printf ("\n");
 
       dn = ksba_cert_get_issuer (cert, 0);
@@ -219,7 +206,7 @@ gpgsm_dump_cert (const char *text, ksba_cert_t cert)
       gpgsm_dump_string (dn);
       ksba_free (dn);
       log_printf ("\n");
-    
+
       dn = ksba_cert_get_subject (cert, 0);
       log_debug ("    subject: ");
       gpgsm_dump_string (dn);
@@ -269,7 +256,7 @@ gpgsm_format_sn_issuer (ksba_sexp_t sn, const char *issuer)
 
 /* Log the certificate's name in "#SN/ISSUERDN" format along with
    TEXT. */
-void 
+void
 gpgsm_cert_log_name (const char *text, ksba_cert_t cert)
 {
   log_info ("%s", text? text:"certificate" );
@@ -314,7 +301,7 @@ parse_dn_part (struct dn_array_s *array, const unsigned char *string)
     {"T",            "2.5.4.12" },
     {"GN",           "2.5.4.42" },
     {"SN",           "2.5.4.4" },
-    {"NameDistinguisher", "0.2.262.1.10.7.20"}, 
+    {"NameDistinguisher", "0.2.262.1.10.7.20"},
     {"ADDR",         "2.5.4.16" },
     {"BC",           "2.5.4.15" },
     {"D",            "2.5.4.13" },
@@ -342,7 +329,7 @@ parse_dn_part (struct dn_array_s *array, const unsigned char *string)
   array->key = p = xtrymalloc (n+10);
   if (!array->key)
     return NULL;
-  memcpy (p, string, n); 
+  memcpy (p, string, n);
   p[n] = 0;
   trim_trailing_spaces (p);
 
@@ -386,7 +373,7 @@ parse_dn_part (struct dn_array_s *array, const unsigned char *string)
             { /* pair */
               s++;
               if (*s == ',' || *s == '=' || *s == '+'
-                  || *s == '<' || *s == '>' || *s == '#' || *s == ';' 
+                  || *s == '<' || *s == '>' || *s == '#' || *s == ';'
                   || *s == '\\' || *s == '\"' || *s == ' ')
                 n++;
               else if (hexdigitp (s) && hexdigitp (s+1))
@@ -401,7 +388,7 @@ parse_dn_part (struct dn_array_s *array, const unsigned char *string)
             return NULL; /* invalid encoding */
           else if (*s == ',' || *s == '=' || *s == '+'
                    || *s == '<' || *s == '>' || *s == ';' )
-            break; 
+            break;
           else
             n++;
         }
@@ -412,7 +399,7 @@ parse_dn_part (struct dn_array_s *array, const unsigned char *string)
       for (s=string; n; s++, n--)
         {
           if (*s == '\\')
-            { 
+            {
               s++;
               if (hexdigitp (s))
                 {
@@ -453,7 +440,7 @@ parse_dn (const unsigned char *string)
       if (!*string)
         break; /* ready */
       if (arrayidx >= arraysize)
-        { 
+        {
           struct dn_array_s *a2;
 
           arraysize += 5;
@@ -492,9 +479,9 @@ parse_dn (const unsigned char *string)
 }
 
 
-/* Print a DN part to STREAM or if STREAM is NULL to FP. */
+/* Print a DN part to STREAM. */
 static void
-print_dn_part (FILE *fp, estream_t stream,
+print_dn_part (estream_t stream,
                struct dn_array_s *dn, const char *key, int translate)
 {
   struct dn_array_s *first_dn = dn;
@@ -513,25 +500,13 @@ print_dn_part (FILE *fp, estream_t stream,
         next:
           if (!dn->done && dn->value && *dn->value)
             {
-              if (stream)
-                {
-                  es_fprintf (stream, "/%s=", dn->key);
-                  if (translate)
-                    es_write_sanitized_utf8_buffer (stream, dn->value,
-                                                    strlen (dn->value),
-                                                    "/", NULL);
-                  else
-                    es_write_sanitized (stream, dn->value, strlen (dn->value),
-                                        "/", NULL);
-                }
+              es_fprintf (stream, "/%s=", dn->key);
+              if (translate)
+                print_utf8_buffer3 (stream, dn->value, strlen (dn->value),
+                                    "/");
               else
-                {
-                  fprintf (fp, "/%s=", dn->key);
-                  if (translate)
-                    print_sanitized_utf8_string (fp, dn->value, '/');
-                  else
-                    print_sanitized_string (fp, dn->value, '/');
-                }
+                es_write_sanitized (stream, dn->value, strlen (dn->value),
+                                    "/", NULL);
             }
           dn->done = 1;
           if (dn > first_dn && dn[-1].multivalued)
@@ -546,68 +521,23 @@ print_dn_part (FILE *fp, estream_t stream,
 /* Print all parts of a DN in a "standard" sequence.  We first print
    all the known parts, followed by the uncommon ones */
 static void
-print_dn_parts (FILE *fp, estream_t stream,
+print_dn_parts (estream_t stream,
                 struct dn_array_s *dn, int translate)
 {
   const char *stdpart[] = {
-    "CN", "OU", "O", "STREET", "L", "ST", "C", "EMail", NULL 
+    "CN", "OU", "O", "STREET", "L", "ST", "C", "EMail", NULL
   };
   int i;
-  
+
   for (i=0; stdpart[i]; i++)
-      print_dn_part (fp, stream, dn, stdpart[i], translate);
+      print_dn_part (stream, dn, stdpart[i], translate);
 
   /* Now print the rest without any specific ordering */
   for (; dn->key; dn++)
-    print_dn_part (fp, stream, dn, dn->key, translate);
+    print_dn_part (stream, dn, dn->key, translate);
 }
 
 
-/* Print the S-Expression in BUF, which has a valid length of BUFLEN,
-   as a human readable string in one line to FP. */
-static void
-pretty_print_sexp (FILE *fp, const unsigned char *buf, size_t buflen)
-{
-  size_t len;
-  gcry_sexp_t sexp;
-  char *result, *p;
-
-  if ( gcry_sexp_sscan (&sexp, NULL, (const char*)buf, buflen) )
-    {
-      fputs (_("[Error - invalid encoding]"), fp);
-      return;
-    }
-  len = gcry_sexp_sprint (sexp, GCRYSEXP_FMT_ADVANCED, NULL, 0);
-  assert (len);
-  result = xtrymalloc (len);
-  if (!result)
-    {
-      fputs (_("[Error - out of core]"), fp);
-      gcry_sexp_release (sexp);
-      return;
-    }
-  len = gcry_sexp_sprint (sexp, GCRYSEXP_FMT_ADVANCED, result, len);
-  assert (len);
-  for (p = result; len; len--, p++)
-    {
-      if (*p == '\n')
-        {
-          if (len > 1) /* Avoid printing the trailing LF. */
-            fputs ("\\n", fp);
-        }
-      else if (*p == '\r')
-        fputs ("\\r", fp);
-      else if (*p == '\v')
-        fputs ("\\v", fp);
-      else if (*p == '\t')
-        fputs ("\\t", fp);
-      else
-        putc (*p, fp);
-    }
-  xfree (result);
-  gcry_sexp_release (sexp);
-}
-
 /* Print the S-Expression in BUF to extended STREAM, which has a valid
    length of BUFLEN, as a human readable string in one line to FP. */
 static void
@@ -654,63 +584,6 @@ pretty_es_print_sexp (estream_t fp, const unsigned char *buf, size_t buflen)
 }
 
 
-
-
-void
-gpgsm_print_name2 (FILE *fp, const char *name, int translate)
-{
-  const unsigned char *s = (const unsigned char *)name;
-  int i;
-
-  if (!s)
-    {
-      fputs (_("[Error - No name]"), fp);
-    }
-  else if (*s == '<')
-    {
-      const char *s2 = strchr ( (char*)s+1, '>');
-      if (s2)
-        {
-          if (translate)
-            print_sanitized_utf8_buffer (fp, s + 1, s2 - (char*)s - 1, 0);
-          else
-            print_sanitized_buffer (fp, s + 1, s2 - (char*)s - 1, 0);
-        }
-    }
-  else if (*s == '(')
-    {
-      pretty_print_sexp (fp, s, gcry_sexp_canon_len (s, 0, NULL, NULL));
-    }
-  else if (!((*s >= '0' && *s < '9')
-             || (*s >= 'A' && *s <= 'Z')
-             || (*s >= 'a' && *s <= 'z')))
-    fputs (_("[Error - invalid encoding]"), fp);
-  else
-    {
-      struct dn_array_s *dn = parse_dn (s);
-      if (!dn)
-        fputs (_("[Error - invalid DN]"), fp);
-      else 
-        {
-          print_dn_parts (fp, NULL, dn, translate);          
-          for (i=0; dn[i].key; i++)
-            {
-              xfree (dn[i].key);
-              xfree (dn[i].value);
-            }
-          xfree (dn);
-        }
-    }
-}
-
-
-void
-gpgsm_print_name (FILE *fp, const char *name)
-{
-  gpgsm_print_name2 (fp, name, 1);
-}
-
-
 /* This is a variant of gpgsm_print_name sending it output to an estream. */
 void
 gpgsm_es_print_name2 (estream_t fp, const char *name, int translate)
@@ -729,8 +602,7 @@ gpgsm_es_print_name2 (estream_t fp, const char *name, int translate)
       if (s2)
         {
           if (translate)
-            es_write_sanitized_utf8_buffer (fp, s + 1, s2 - (char*)s - 1,
-                                            NULL, NULL);
+            print_utf8_buffer (fp, s + 1, s2 - (char*)s - 1);
           else
             es_write_sanitized (fp, s + 1, s2 - (char*)s - 1, NULL, NULL);
         }
@@ -749,9 +621,9 @@ gpgsm_es_print_name2 (estream_t fp, const char *name, int translate)
 
       if (!dn)
         es_fputs (_("[Error - invalid DN]"), fp);
-      else 
+      else
         {
-          print_dn_parts (NULL, fp, dn, translate);          
+          print_dn_parts (fp, dn, translate);
           for (i=0; dn[i].key; i++)
             {
               xfree (dn[i].key);
@@ -771,7 +643,7 @@ gpgsm_es_print_name (estream_t fp, const char *name)
 
 
 /* A cookie structure used for the memory stream. */
-struct format_name_cookie 
+struct format_name_cookie
 {
   char *buffer;         /* Malloced buffer with the data to deliver. */
   size_t size;          /* Allocated size of this buffer. */
@@ -799,7 +671,7 @@ format_name_writer (void *cookie, const void *buffer, size_t size)
   else if (c->len + size < c->len)
     {
       p = NULL;
-      errno = ENOMEM;
+      gpg_err_set_errno (ENOMEM);
     }
   else if (c->size < c->len + size)
     {
@@ -817,12 +689,12 @@ format_name_writer (void *cookie, const void *buffer, size_t size)
       c->error = errno;
       xfree (c->buffer);
       c->buffer = NULL;
-      errno = c->error;
+      gpg_err_set_errno (c->error);
       return -1;
     }
   memcpy (p + c->len, buffer, size);
   c->len += size;
-  p[c->len] = 0; /* Terminate string. */ 
+  p[c->len] = 0; /* Terminate string. */
 
   return (ssize_t)size;
 }
@@ -847,8 +719,8 @@ gpgsm_format_name2 (const char *name, int translate)
   if (!fp)
     {
       int save_errno = errno;
-      log_error ("error creating memory stream: %s\n", strerror (errno));
-      errno = save_errno;
+      log_error ("error creating memory stream: %s\n", strerror (save_errno));
+      gpg_err_set_errno (save_errno);
       return NULL;
     }
   gpgsm_es_print_name2 (fp, name, translate);
@@ -856,7 +728,7 @@ gpgsm_format_name2 (const char *name, int translate)
   if (cookie.error || !cookie.buffer)
     {
       xfree (cookie.buffer);
-      errno = cookie.error;
+      gpg_err_set_errno (cookie.error);
       return NULL;
     }
   return cookie.buffer;
@@ -882,7 +754,7 @@ gpgsm_fpr_and_name_for_status (ksba_cert_t cert)
   fpr = gpgsm_get_fingerprint_hexstring (cert, GCRY_MD_SHA1);
   if (!fpr)
     return NULL;
-  
+
   name = ksba_cert_get_subject (cert, 0);
   if (!name)
     {
@@ -965,23 +837,22 @@ gpgsm_format_keydesc (ksba_cert_t cert)
                          "created %s, expires %s.\n" ),
                        subject? subject:"?",
                        sn? sn: "?",
-                       gpgsm_get_short_fingerprint (cert),
+                       gpgsm_get_short_fingerprint (cert, NULL),
                        created, expires);
-  
+
   i18n_switchback (orig_codeset);
-  
+
   if (!name)
     {
       xfree (subject);
       xfree (sn);
       return NULL;
     }
-  
+
   xfree (subject);
   xfree (sn);
 
   buffer = percent_plus_escape (name);
-  xfree (name); 
+  xfree (name);
   return buffer;
 }
-
